VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Complex"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public Re As Double
Public Im As Double
' See "Complex number raised to a complex power"
' at http://home.att.net/~srschmitt/complexnumbers.html.
' x^y = r^c * e^(-d * theta) *
'      [cos(c * theta + d * ln(r)) +
'       sin(c * theta + d * ln(r)) * i]
' Where
'       x = a + b * i
'       y = c + d * i
'       r = Sqrt(a^2 + b^2)
'       theta = Atan2(b, a)
Public Function ToThePowerOf(ByVal x As Complex) As Complex
Const E As Double = 2.71828182845905

Dim r As Double
Dim theta As Double
Dim factor As Double
Dim new_angle As Double

    r = Sqr(Re * Re + Im * Im)
    theta = Atan2(Im, Re)
    factor = (r ^ x.Re) * (E ^ (-x.Im * theta))
    new_angle = x.Re * theta + x.Im * Log(r)
    Set ToThePowerOf = NewComplex( _
        factor * Cos(new_angle), _
        factor * Sin(new_angle))
End Function
Public Function Minus(ByVal x As Complex) As Complex
    Set Minus = NewComplex(Re - x.Re, Im - x.Im)
End Function

Public Function Clone() As Complex
    Set Clone = NewComplex(Re, Im)
End Function
Public Sub Initialize(ByVal new_re As Double, ByVal new_im As Double)
    Re = new_re
    Im = new_im
End Sub


Public Function MagnitudeSquared() As Double
    MagnitudeSquared = Re * Re + Im * Im
End Function
Public Function Magnitude() As Double
    Magnitude = Sqr(MagnitudeSquared())
End Function
Public Function Times(ByVal x As Complex) As Complex
    Set Times = NewComplex(Re * x.Re - Im * x.Im, Re * x.Im + Im * x.Re)
End Function
Public Function TimesComponents(ByVal new_re As Double, ByVal new_im As Double) As Complex
    Set TimesComponents = NewComplex(Re * new_re - Im * new_im, Re * new_im + Im * new_re)
End Function
Public Function Plus(ByVal x As Complex) As Complex
    Set Plus = NewComplex(Re + x.Re, Im + x.Im)
End Function
Public Function PlusComponents(ByVal new_re As Double, ByVal new_im As Double) As Complex
    Set PlusComponents = NewComplex(Re + new_re, Im + new_im)
End Function
Public Function DividedBy(ByVal denominator As Complex) As Complex
Dim conjugate As Complex
Dim numerator As Complex

    Set conjugate = denominator.Clone
    conjugate.Im = -conjugate.Im

    Set denominator = denominator.Times(conjugate)
    Set numerator = Me.Times(conjugate)
    numerator.Re = numerator.Re / denominator.Re
    numerator.Im = numerator.Im / denominator.Re

    Set DividedBy = numerator
End Function
Public Function IsCloseTo(ByVal x As Complex) As Boolean
    IsCloseTo = (Abs(Re - x.Re) + Abs(Im - x.Im) < 0.001)
End Function
Public Function ToString() As String
    If Im < 0 Then
        ToString = Format$(Re) & " - " & Format$(Abs(Im)) & "i"
    Else
        ToString = Format$(Re) & " + " & Format$(Im) & "i"
    End If
End Function
